import Echo from 'laravel-echo';
import socketio from 'socket.io-client';

/**
 * EchoRedux подключает laravel-echo к redux.
 *    
*/


//Дефолтные типы экшенов которые заменятся если передать новый тип экшена в соотвествующие функции
const SOCKET_LEAVE = 'SOCKET:LEAVE_CHANNEL';
const SOCKET_STOP_LISTENING = 'SOCKET:STOP_LISTENING';
const SOCKET_EVENT = 'SOCKET:NEW_EVENT';

//Типы экшенов которые заложены в сокет по дефоллту (лучше не менять,а запомнить);
const SOCKET_CONNECTED = 'SOCKET:CONNECTED';
const SOCKET_DISCONNECTED = 'SOCKET:DISCONNECTED';
const SOCKET_RECONNECTING = 'SOCKET:RECONNECTING';
const SOCKET_RECONNECTED = 'SOCKET:RECONNECTED';

class EchoRedux {

    constructor() {
        this.store = {};
        this.echo;
        this.debug = false;
    }
    
    _disaptchAction(actionType, payload){
        return this.store.dispatch({ type: actionType, payload })
    }

    _debugConnected() {
        if (this.debug)
            console.log('Echo connected: ', this.echo.socketId);
        this._disaptchAction(SOCKET_CONNECTED);
    }

    _debugDisconnected() {
        if (this.debug)
            console.log('Echo disconnected.');
        this._disaptchAction(SOCKET_DISCONNECTED);
    }

    _debugReconnecting(number) {
        if (this.debug)
            console.log('Echo reconnecting: ', number);
        this._disaptchAction(SOCKET_RECONNECTING);
    }

    _debugReconnected() {
        if (this.debug)
            console.log('Echo reconnected.');
        this._disaptchAction(SOCKET_RECONNECTED);
    }

    _subscribing(channel, event, actionType) {
        if (Array.isArray(event)) {
            if (Array.isArray(actionType)) {
                event.forEach((item, index) => {
                    channel.listen(item, event => this._disaptchAction(actionType[index] || SOCKET_EVENT, { channelName: channel, event: event}));
                })
            } else {
                throw new Error('Actions types is not array!')
            }
        } else if (Array.isArray(actionType)) {
            throw new Error('Actions types not be array!')
        } else {
            channel.listen(event, event => this._disaptchAction(actionType || SOCKET_EVENT, { channelName: channel, event: event}));
        }
   }

    init(config) {
        if (!this.echo) {

            const _config = config;
            this.debug = _config.debug;

            delete _config.debug;

            if (_config.store) {
                this.store = _config.store;
                delete _config.store;
            
            } else {
                throw new Error('Store is undefined!')
            }
            this.echo = new Echo({
                host: 'localhost:6001',
                broadcaster: 'socket.io',
                client: socketio,
                encrypted: true,
                ..._config
            });
      
            this.echo.connector.socket.on('connect', () => {
                this._debugConnected();
            })
            this.echo.connector.socket.on('disconnect', () => {
                this._debugDisconnected();
            });
            this.echo.connector.socket.on('reconnecting', (attemptNumber) => {
                this._debugReconnecting(attemptNumber)
            });  
            this.echo.connector.socket.on('reconnect', () => {
                this._debugReconnected()
            });  
            
        } else {
            throw new Error('Echo alredy initialization!')
        }
    }

    subscribePrivate(channel, event, actionType) {
        if (this.echo) {
            const _channel = this.echo.private(channel);
            this._subscribing(_channel, event, actionType);
        } else {
            throw new Error('Echo is not initialization!')
        }
    }
    
    subscribe(channel, event, actionType) {
        if (this.echo) {
            const _channel = this.echo.channel(channel);
            this._subscribing(_channel, event, actionType);
        } else {
            throw new Error('Echo is not initialization!')
        }
    }

    unsubscribeEvent(channel, event, actionType) {
        if (this.echo) {
            const _channel = this.echo.private(channel);
            if (Array.isArray(event)) {
                if (Array.isArray(actionType)) {
                    event.forEach((item, index) => {
                        _channel.stopListening(item);
                        this._disaptchAction(actionType[index] || SOCKET_STOP_LISTENING, {channelName: channel, eventName: event})
                    })
                } else {
                    throw new Error('Actions types is not array!')
                }
            } else if (Array.isArray(actionType)) {
                throw new Error('Actions types not be array!')
            } else {
                _channel.stopListening(event);
                this._disaptchAction(actionType || SOCKET_STOP_LISTENING, {channelName: channel, eventName: event})
            }
        } else {
            throw new Error('Echo is not initialization!')
        }    
    }

    getChannel(channel) {
        const _channel = this.echo.channel(channel);
        if (!_channel) {
            console.warn('Not subscribed on channel', _channel.name);
            return;
        }
        return _channel;
    }

    getAllChannels() {
        return this.echo.connector.channels;
    }

    getEvents(channel) {
        const _channel = this.echo.channel(channel);
        if (!_channel) {
            console.warn('Not subscribed on channel', _channel.name);
            return;
        }
        return _channel.events;
    }

    leaveChannel(channel, actionType) {
        this.echo.leave(channel);
        this._disaptchAction(actionType || SOCKET_LEAVE, { channelName: channel})
    }

}

export default new EchoRedux;